{% extends "layout.html" %}


{% block body %}
    <header id="header" class="row">
      <div class="col-xs-12 col-sm-5 offset-md-2">
            <h2>{{ _("Manage your shared <br>expenses, easily") }}</h2>
            {% if is_demo_project_activated %}
            <a href="{{ url_for(".demo") }}" class="tryout btn">{{ _("Try out the demo")  }}</a>
            {% endif %}
        </div>
        <div class="col-xs-12 col-sm-4">
            <p class="additional-content">{{ _("You're sharing a house?") }}<br /> {{ _("Going on holidays with friends?") }}<br /> {{ _("Simply sharing money with others?") }} <br /><strong>{{ _("We can help!") }}</strong></p>
        </div>
    </header>

    <main class="row home">
        <div class="col-xs-12 col-sm-5 col-md-4 offset-md-2">
            <form id="authentication-form" class="form-horizontal" action="{{ url_for(".authenticate") }}" method="post">
                <fieldset class="form-group">
                    <legend>{{ _("Log to an existing project") }}...</legend>
                    {{ forms.authenticate(auth_form, home=True) }}
                </fieldset>
                <div class="controls">
                    <button class="btn" type="submit">{{ _("log in") }}</button>
                    <a class="password-reminder" href="{{ url_for(".remind_password") }}">{{ _("can't remember your password?") }}</a>
                </div>
            </form>
        </div>
        <div class="col-xs-12 col-sm-5 col-md-3 offset-sm-1">
            {% if is_public_project_creation_allowed %}
            <form id="creation-form" class="form-horizontal" action="{{ url_for(".create_project") }}" method="post">
                <fieldset class="form-group">
                    <legend>...{{ _("or create a new one") }}</legend>
                    {{ forms.create_project(project_form, home=True) }}
                </fieldset>
                <div class="controls">
                    <button class="btn" type="submit">{{ _("let's get started") }}</button>
                </div>
            </form>
            {% else %}
            <a href="{{ url_for(".create_project") }}">...{{ _("or create a new one") }}</a>
            {% endif %}
        </main>
    </div>
{% endblock %}

{% block js %}

$('#creation-form #password').tooltip({
    title: '{{ _("This access code will be sent to your friends. It is stored as-is by the server, so don\\'t reuse a personal password!") }}',
    trigger: 'focus',
    placement: 'right'
});

{% endblock %}
