{% extends "layout.html" %}

{% block js %}
    $('#delete-project').click(function ()
    {
        $(this).html("<a style='color:red; ' href='{{ url_for('.delete_project') }}' >{{_("you sure?")}}</a>");
    });
{% endblock %}

{% block content %}
<h2>{{ _("Edit this project") }}</h2>
<form class="form-horizontal" method="post">
{{ forms.edit_project(edit_form) }}
</form></br>
<h2>{{ _("Download this project's data") }}</h2>
<form class="form-horizontal" method="post">
{{ forms.export_project(export_form) }}
</form>
{% endblock %}
