{% extends "layout.html" %}

{% block body %}
<div class="row" style="height: 100%">
<aside id="sidebar" class="sidebar col-xs-12 col-md-3 " style="height: 100%">
  {% block sidebar %}{% endblock %}
</aside>

<main class="offset-md-3 col-xs-12 col-md-9">
  {% block content %}{% endblock %}
</main>

</div>
{% endblock %}
