{% extends "layout.html" %}

{% block content %}
<h2>{{ _("Invite people to join this project") }}</h2>
<p>{{ _("Specify a (comma separated) list of email adresses you want to notify about the
creation of this budget management project and we will send them an email for you.") }}</p>
<p>{{ _("If you prefer, you can share the project identifier and the shared
password by other communication means. Or even directly share the following link:") }}</br>
<a href="{{ url_for(".authenticate", _external=True, token=g.project.generate_token()) }}">
    {{ url_for(".authenticate", _external=True, token=g.project.generate_token()) }}
</a>
</p>

{% include "display_errors.html" %}
<form class="invites form-horizontal" method="post" accept-charset="utf-8">
    {{ forms.invites(form) }}
</form>
{% endblock %}
