{% import "forms.html" as forms %}
<!DOCTYPE html>
<html>
<head>
    <title>{{ _("Account manager") }}{% block title %}{% endblock %}</title>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel=stylesheet type=text/css href="{{ url_for("static", filename='css/main.css') }}">
    <script src="{{ url_for("static", filename="js/jquery-3.1.1.min.js") }}"></script>
    <script src="{{ url_for("static", filename="js/ihatemoney.js") }}"></script>
    <script src="{{ url_for("static", filename="js/tether.min.js") }}"></script>
    <script src="{{ url_for("static", filename="js/bootstrap.min.js") }}"></script>
    {% block head %}{% endblock %}
    <script type="text/javascript" charset="utf-8">
        $(document).ready(function(){
            setTimeout(function(){
                $(".flash").fadeOut("slow", function () {
                    $(".flash").remove();
                });
            }, 4000);

            $('.dropdown-toggle').dropdown();

            {% block js %}{% endblock %}
        });
    </script>
</head>
<body>
  <div class="container">
    <nav class="navbar navbar-toggleable-sm fixed-top navbar-inverse bg-inverse">
      <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarToggler" aria-controls="navbarToggler" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
      </button>
      <div class="collapse navbar-collapse" id="navbarToggler">
        <h1><a class="navbar-brand" href="{{ url_for(".home") }}">#! money?</a></h1>
        <ul class="navbar-nav ml-auto mr-auto">
          {% if g.project %}
          {% block navbar %}
          <li class="nav-item{% if current_view == 'list_bills' %} active{% endif %}"><a class="nav-link" href="{{ url_for(".list_bills") }}">{{ _("Bills") }}</a></li>
          <li class="nav-item{% if current_view == 'settle_bill' %} active{% endif %}"><a class="nav-link" href="{{ url_for(".settle_bill") }}">{{ _("Settle") }}</a></li>
          <li class="nav-item{% if current_view == 'statistics' %} active{% endif %}"><a class="nav-link" href="{{ url_for(".statistics") }}">{{ _("Statistics") }}</a></li>
          {% endblock %}
          {% endif %}
        </ul>
        <ul class="navbar-nav secondary-nav">
          {% if g.project %}
          <li class="nav-item dropdown">
            <a href="#" class="nav-link dropdown-toggle" id="navbarDropdownMenuLink" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"><strong>{{ g.project.name }}</strong> {{ _("options") }} <b class="caret"></b></a>
            <ul class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
              <li><a class="dropdown-item" href="{{ url_for(".edit_project") }}">{{ _("Project settings") }}</a></li>
              <li class="dropdown-divider"></li>
              {% for id, name in session['projects'] %}
              {% if id != g.project.id %}
              <li><a class="dropdown-item" href="{{ url_for(".list_bills", project_id=id) }}">{{ _("switch to") }} {{ name }}</a></li>
              {% endif %}
              {% endfor %}
              <li><a class="dropdown-item" href="{{ url_for(".create_project") }}">{{ _("Start a new project") }}</a></li>
              <li class="dropdown-divider"></li>
              <li><a class="dropdown-item" href="{{ url_for(".exit") }}">{{ _("Logout") }}</a></li>
            </ul>
          </li>
          {% endif %}
          <li class="nav-item{% if g.lang == "fr" %} active{% endif %}"><a class="nav-link" href="{{ url_for(".change_lang", lang="fr") }}">fr</a></li>
          <li class="nav-item{% if g.lang == "en" %} active{% endif %}"><a class="nav-link" href="{{ url_for(".change_lang", lang="en") }}">en</a></li>
          {% if g.show_admin_dashboard_link %}
          <li class="nav-item{% if request.url_rule.endpoint == "main.dashboard" %} active{% endif %}"><a class="nav-link" href="{{ url_for(".dashboard") }}">{{ _("Dashboard") }}</a></li>
          {% endif %}
        </ul>
      </div>
    </nav>
  </div>

<div class="container-fluid">
{% block body %}
    <main class="content offset-1 col-10">
    {% block content %}{% endblock %}
    </main>
</div>
{% endblock %}

<div class="messages">
    {% for message in get_flashed_messages() %}
    <div class="flash alert alert-success">{{ message }}</div>
    {% endfor %}
</div>

{% block footer %}
<footer>
    <p><a href="https://github.com/spiral-project/ihatemoney">{{ _("This is a free software") }}</a>, {{ _("you can contribute and improve it!") }}</p>
</footer>
{% endblock %}

</body>
</html>
